#!/bin/sh
set -e

if [ "$#" -ne 1 ]; then
    echo 'Need mail for letsencrypt.'
    false
fi
mail="$1"

config_tree_prefix="${HOME}/config/buster"
echo "postfix postfix/main_mailer_type string 'Internet Site'" | debconf-set-selections
echo "postfix postfix/mailname string $(hostname -f)" | debconf-set-selections
./install_for_target.sh mail
./copy_dirtree.sh "${config_tree_prefix}/etc_files" "" mail
nft -f /etc/nftables.conf

cat "${config_tree_prefix}/other_files/append_postfix_main.cf" >> /etc/postfix/main.cf
cat "${config_tree_prefix}/other_files/append_postfix_master.cf" >> /etc/postfix/main.cf
cat "${config_tree_prefix}/other_files/append_opendkim.conf" >> /etc/opendkim.conf

# Set up letsencrypt certificate.  We need this for STARTTLS on port
# 25/SMTP (some mail servers refuse delivering mails here if no
# STARTTLS available) and transport-layer TLS on port 465 (for
# user-to-server SMTPS)
# TODO: Is it auto-renewed?
certbot certonly --standalone --agree-tos --no-eff-email -m "${mail}" -d "$(hostname -f)"

# OpenDKIM setup.
selector=$(hostname)$(date +%Y%m%d)
#opendkim-genkey -D /etc/dkimkeys -s "${selector}"
opendkim-genkey -d "$(hostname -f)" -D /etc/dkimkeys -s "${selector}"
sed -i "s/REPLACE_hostname_ECALPER/$(hostname -f)/g" /etc/opendkim.conf
sed -i "s/REPLACE_selector_ECALPER/${selector}/g" /etc/opendkim.conf

# Dovecot sieve filtering via LMTP.  Without this, mail only gets
# delivered to /var/mail/….
cp "${config_tree_prefix}/other_files/dovecot.sieve" /home/plom/.dovecot.sieve
chown plom:plom /home/plom/.dovecot.sieve

# To allow IMAPS access.
echo "ssl_cert = </etc/letsencrypt/live/$(hostname -f)/fullchain.pem" > /etc/dovecot/conf.d/99-ssl-certs.conf
echo "ssl_key = </etc/letsencrypt/live/$(hostname -f)/privkey.pem" >> /etc/dovecot/conf.d/99-ssl-certs.conf
password=$(pwgen -s 100 1)
#echo 'mail_privileged_group = mail' >> /etc/dovecot/conf.d/99-mail.conf
echo "plom:${password}" | chpasswd

service opendkim restart
service postfix restart
service dovecot restart

echo "To put into DNS:"
cat "/etc/dkimkeys/${selector}.txt"
echo "If subdomain, append .subdomain to _domainkeys!"
echo "Also ensure DMARC record of 'v=DMARC1; p=none; rua=mailto:plom+dmarc@plomlompom.com;' as TXT entry at _dmarc or, if subdomain, _dmarc.subdomain"
echo "Also ensure SPF record of 'v=spf1 mx -all' at @ or subdomain"
echo "Also ensure reverse DNS lookup for our IP points to $(hostname -f)"
echo "Also ensure MX record of priority 10 for @ or subdomain pointing to $(hostname -f)"
echo "IMAPS password for user plom is: ${password}"

# todo just for proper mail /sending/:
# * figure out /etc/mailname ("used by the Mail Transfer Agent (i.e. mail server) to know its own hostname", "will contain the portion after the username and @ (at) sign for email addresses of users on the machine", "Postfix sets myorigin=/etc/mailname. Myorigin is appended (with an @) to any bare name in any address field. During (re)configure, if /etc/mailname exists, it is used as the default value for myorigin in debconf dialogs. Changing it from the default results in postinst rewriting /etc/mailname to the new value.") – figure out if it is actually used?
# * figure out /etc/postfix/main.cf
# - myorigin ("specifies the domain that appears in mail that is posted on this machine. The default is to use the local machine name, $myhostname, which defaults to the name of the machine. Unless you are running a really small site, you probably want to change that into $mydomain, which defaults to the parent domain of the machine name")
# - myhostname ("The internet hostname of this mail system. The default is to use the fully-qualified domain name (FQDN) from gethostname(), or to use the non-FQDN result from gethostname() and append ".$mydomain". $myhostname is used as a default value for many other configuration parameters.")
# - mydomain ("The internet domain name of this mail system. The default is to use $myhostname minus the first component, or "localdomain" (Postfix 2.3 and later). $mydomain is used as a default value for many other configuration parameters.")
# - relay_domains ("What destination domains (and subdomains thereof) this system will relay mail to")
# - mydestination ("The list of domains that are delivered via the $local_transport mail delivery transport.")
# * figure out /etc/postfix/master.cf (configures what postfix daemons run in what way?)
# * how to check IP safety
# https://talosintelligence.com/reputation_center/lookup?search=$IP
# http://www.anti-abuse.org/multi-rbl-check-results/?host=
# https://www.dnsbl.info/dnsbl-database-check.php
# note that none of these catch the IPs that gmx etc. reject

# other stuff:
# * figure out /etc/aliases (maps whom what is sent to – just re-use old core.plomlompom.com one? – also, run newaliases to generate /etc/aliases.db)
# * more /etc/postfix/main.cf
# - smtpd_recipient_restrictions
# - smtpd_helo_restrictions
# - smtpd_client_restrictions
#
# for using SMTP server remotely:
# - SASL mechanism, may use dovecot for that (smtpd_sasl_type, smtpd_sasl_path)
#   - see https://wiki2.dovecot.org/HowTo/PostfixAndDovecotSASL
# - SASL is not a protocol but an abstraction layer to some auth mechanism
# - SSL works on transport layer
# - wild guess: TLS/SSL is used to authenticate /the server/ to the client, while SASL is used to identify /the client/ to the server
#   - then it should be possible to do SASL without TLS/STARTTLS first? (experiment)
#   - the telnet test should offer AUTH then without doing STARTTLS first
